@@ -149,6 +149,8 @@ class UICallbacks:
         socket.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("black"))
         socket.set_events(gtk.gdk.ALL_EVENTS_MASK)
         socket.set_flags(gtk.CAN_FOCUS)
+        # set max Hatari window size = desktop size
+        self.config.set_desktop_size(gtk.gdk.screen_width(), gtk.gdk.screen_height())
         # set initial embedded hatari size
         width, height = self.config.get_window_size()
         socket.set_size_request(width, height)
@@ -439,7 +441,6 @@ class UIActions:
         ("compatibility", None, "Hatari compatibility list", None, None, self.help.view_hatari_compatibility),
         ("release", None, "Hatari release notes", None, None, self.help.view_hatari_releasenotes),
         ("todo", None, "Hatari TODO", None, None, self.help.view_hatari_todo),
-        ("bugs", None, "Report a bug", None, None, self.help.view_hatari_bugs),
         ("mails", None, "Hatari mailing lists", None, None, self.help.view_hatari_mails),
         ("changes", None, "Latest Hatari changes", None, None, self.help.view_hatari_repository),
         ("authors", None, "Hatari authors", None, None, self.help.view_hatari_authors),
@@ -617,7 +618,7 @@ class UIActions:
         ("Devices", ("display", "floppy", "harddisk", "joystick", "machine", "device", "sound")),
         ("Configuration", ("path", None, "lconfig", "sconfig")),
         ("Debug", ("debug", "trace")),
-        ("Help", ("manual", "compatibility", "release", "todo", None, "bugs", "mails", "changes", None, "authors", "hatari", "hatariui", "about",))
+        ("Help", ("manual", "compatibility", "release", "todo", None, "mails", "changes", None, "authors", "hatari", "hatariui", "about",))
         )
         bar = gtk.MenuBar()
 
