@@ -104,7 +104,7 @@ class UIHelp:
             return path
         # if not, point to latest Hatari HG version docs
         print("WARNING: Hatari manual not found at:", path + "manual.html")
-        return "http://hg.berlios.de/repos/hatari/raw-file/tip/doc/"
+        return "http://hg.tuxfamily.org/mercurialroot/hatari/hatari/raw-file/tip/doc/"
 
     def set_mainwin(self, widget):
         self.mainwin = widget
@@ -134,20 +134,17 @@ class UIHelp:
     def view_hatari_todo(self, dummy=None):
         self.view_url(self._path + "todo.txt", "Hatari TODO items")
 
-    def view_hatari_bugs(self, dummy=None):
-        self.view_url("http://developer.berlios.de/bugs/?group_id=10436", "Hatari bug tracker")
-
     def view_hatari_mails(self, dummy=None):
-        self.view_url("http://developer.berlios.de/mail/?group_id=10436", "Hatari mailing lists")
+        self.view_url("http://hatari.tuxfamily.org/contact.html", "Hatari mailing lists")
 
     def view_hatari_repository(self, dummy=None):
-        self.view_url("http://hg.berlios.de/repos/hatari", "latest Hatari changes")
+        self.view_url("http://hg.tuxfamily.org/mercurialroot/hatari/hatari", "latest Hatari changes")
 
     def view_hatari_authors(self, dummy=None):
         self.view_url(self._path + "authors.txt", "Hatari authors")
 
     def view_hatari_page(self, dummy=None):
-        self.view_url("http://hatari.berlios.de/", "Hatari home page")
+        self.view_url("http://hatari.tuxfamily.org/", "Hatari home page")
 
     def view_hatariui_page(self, dummy=None):
         self.view_url("http://koti.mbnet.fi/tammat/hatari/hatari-ui.shtml", "Hatari UI home page")
