@@ -59,17 +59,22 @@ static bool bRemotePaused;
 static bool Control_InsertKey(const char *event)
 {
 	const char *key = NULL;
-	bool press;
+	bool up, down;
 
 	if (strncmp(event, "keypress ", 9) == 0) {
 		key = &event[9];
-		press = true;
-	} else if (strncmp(event, "keyrelease ", 11) == 0) {
-		key = &event[11];
-		press = false;
+		down = up = true;
+	} else if (strncmp(event, "keydown ", 8) == 0) {
+		key = &event[8];
+		down = true;
+		up = false;
+	} else if (strncmp(event, "keyup ", 6) == 0) {
+		key = &event[6];
+		down = false;
+		up = true;
 	}
 	if (!(key && key[0])) {
-		fprintf(stderr, "ERROR: event '%s' contains no key press/release\n", event);
+		fprintf(stderr, "ERROR: '%s' contains no key press/down/up event\n", event);
 		return false;
 	}
 	if (key[1]) {
@@ -78,16 +83,28 @@ static bool Control_InsertKey(const char *event)
 		int keycode = strtol(key, &endptr, 0);
 		/* not a valid number or keycode is out of range? */
 		if (*endptr || keycode < 0 || keycode > 255) {
-			fprintf(stderr, "ERROR: '%s' is not valid key code, got %d\n",
+			fprintf(stderr, "ERROR: '%s' is not valid key scancode, got %d\n",
 				key, keycode);
 			return false;
 		}
-		IKBD_PressSTKey(keycode, press);
+		if (down) {
+			IKBD_PressSTKey(keycode, true);
+		}
+		if (up) {
+			IKBD_PressSTKey(keycode, false);
+		}
 	} else {
-		Keymap_SimulateCharacter(key[0], press);
+		if (down) {
+			Keymap_SimulateCharacter(key[0], true);
+		}
+		if (up) {
+			Keymap_SimulateCharacter(key[0], false);
+		}
 	}
-	fprintf(stderr, "Simulated %s key %s\n",
-		key, (press?"press":"release"));
+#if 0
+	fprintf(stderr, "Simulated key %s of %d\n",
+		(down? (up? "press":"down") :"up"), key);
+#endif
 	return true;
 }
 
@@ -106,11 +123,11 @@ static bool Control_InsertEvent(const char *event)
 		Keyboard.LButtonDblClk = 1;
 		return true;
 	}
-	if (strcmp(event, "rightpress") == 0) {
+	if (strcmp(event, "rightdown") == 0) {
 		Keyboard.bRButtonDown |= BUTTON_MOUSE;
 		return true;
 	}
-	if (strcmp(event, "rightrelease") == 0) {
+	if (strcmp(event, "rightup") == 0) {
 		Keyboard.bRButtonDown &= ~BUTTON_MOUSE;
 		return true;
 	}
@@ -118,13 +135,17 @@ static bool Control_InsertEvent(const char *event)
 		return true;
 	}
 	fprintf(stderr, "ERROR: unrecognized event: '%s'\n", event);
-	fprintf(stderr, "Supported events are:\n");
-	fprintf(stderr, "- doubleclick\n");
-	fprintf(stderr, "- rightpress\n");
-	fprintf(stderr, "- rightrelease\n");
-	fprintf(stderr, "- keypress <character>\n");
-	fprintf(stderr, "- keyrelease <character>\n");
-	fprintf(stderr, "<character> can be either a single ASCII char or keycode.\n");
+	fprintf(stderr,
+		"Supported mouse button and key events are:\n"
+		"- doubleclick\n"
+		"- rightdown\n"
+		"- rightup\n"
+		"- keypress <key>\n"
+		"- keydown <key>\n"
+		"- keyup <key>\n"
+		"<key> can be either a single ASCII character or an ST scancode\n"
+		"(e.g. space has scancode of 57 and enter 28).\n"
+		);
 	return false;	
 }
 
