@@ -142,8 +142,7 @@ struct dma_s {
 	Uint16 FIFO_Pos;		/* from 0 to DMASND_FIFO_SIZE-1 */
 	Uint16 FIFO_NbBytes;		/* from 0 to DMASND_FIFO_SIZE */
 
-	Sint16 FrameMono;		/* latest values read from the FIFO */
-	Sint16 FrameLeft;
+	Sint16 FrameLeft;		/* latest values read from the FIFO */
 	Sint16 FrameRight;
 };
 
@@ -154,7 +153,8 @@ bool	DmaInitSample = false;
 struct microwire_s {
 	Uint16 data;			/* Microwire Data register */
 	Uint16 mask;			/* Microwire Mask register */
-	int mwTransferSteps;		/* Microwire shifting counter */
+	Uint16 mwTransferSteps;		/* Microwire shifting counter */
+	Uint16 pendingCyclesOver;	/* Number of delayed cycles for the interrupt */
 	Uint16 mixing;			/* Mixing command */
 	Uint16 bass;			/* Bass command */
 	Uint16 treble;			/* Treble command */
@@ -252,7 +252,6 @@ void DmaSnd_Reset(bool bCold)
 
 	dma.FIFO_Pos = 0;
 	dma.FIFO_NbBytes = 0;
-	dma.FrameMono = 0;
 	dma.FrameLeft = 0;
 	dma.FrameRight = 0;
 
@@ -271,6 +270,7 @@ void DmaSnd_Reset(bool bCold)
 	DmaSnd_Init_Bass_and_Treble_Tables();
 
 	microwire.mwTransferSteps = 0;
+	microwire.pendingCyclesOver = 8;
 }
 
 /*-----------------------------------------------------------------------*/
@@ -496,11 +496,14 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 			switch (microwire.mixing) {
 				case 1:
 					/* YM2149 */
+					MixBuffer[nBufIdx][1]  = MixBuffer[nBufIdx][0] + dma.FrameRight * -((256*3/4)/4)/4;	
+					MixBuffer[nBufIdx][0] += dma.FrameLeft * -((256*3/4)/4)/4;	
 					break;
 				default:
 					/* YM2149 - 12dB */
 					MixBuffer[nBufIdx][0] /= 4;
-					MixBuffer[nBufIdx][1] = MixBuffer[nBufIdx][0];	
+					MixBuffer[nBufIdx][1]  = MixBuffer[nBufIdx][0] + dma.FrameRight * -((256*3/4)/4)/4;	
+					MixBuffer[nBufIdx][0] += dma.FrameLeft * -((256*3/4)/4)/4;	
 					break;
 			}
 		}
@@ -526,8 +529,8 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 			if ( DmaInitSample )
 			{
 				MonoByte = DmaSnd_FIFO_PullByte ();
-				dma.FrameMono  = DmaSnd_LowPassFilterLeft( (Sint16)MonoByte );
-				/* No-Click */   DmaSnd_LowPassFilterRight( (Sint16)MonoByte );
+				dma.FrameLeft  = DmaSnd_LowPassFilterLeft( (Sint16)MonoByte );
+				dma.FrameRight = DmaSnd_LowPassFilterRight( (Sint16)MonoByte );
 				DmaInitSample = false;
 			}
 
@@ -536,16 +539,16 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 			switch (microwire.mixing) {
 				case 1:
 					/* DMA and YM2149 mixing */
-					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + dma.FrameMono * -((256*3/4)/4)/4;
+					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
 					break;
 				case 2:
 					/* DMA sound only */
-					MixBuffer[nBufIdx][0] = dma.FrameMono * -((256*3/4)/4)/4;
+					MixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
 					break;
 				default:
 					/* DMA and (YM2149 - 12dB) mixing */
 					/* instead of 16462 (-12dB), we approximate by 16384 */
-					MixBuffer[nBufIdx][0] = (dma.FrameMono * -((256*3/4)/4)/4) +
+					MixBuffer[nBufIdx][0] = (dma.FrameLeft * -((256*3/4)/4)/4) +
 								(((Sint32)MixBuffer[nBufIdx][0] * 16384)/65536);
 					break;
 			}
@@ -558,8 +561,8 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 			while ( n > 0 )						/* pull as many bytes from the FIFO as needed */
 			{
 				MonoByte = DmaSnd_FIFO_PullByte ();
-				dma.FrameMono  = DmaSnd_LowPassFilterLeft( (Sint16)MonoByte );
-				/* No-Click */   DmaSnd_LowPassFilterRight( (Sint16)MonoByte );
+				dma.FrameLeft  = DmaSnd_LowPassFilterLeft( (Sint16)MonoByte );
+				dma.FrameRight = DmaSnd_LowPassFilterRight( (Sint16)MonoByte );
 				n--;
 			}
 			frameCounter_float &= 0xffffffff;			/* only keep the fractional part */
@@ -636,8 +639,8 @@ static void DmaSnd_Apply_LMC(int nMixBufIdx, int nSamplesToGenerate)
 	/* Apply LMC1992 sound modifications (Left, Right and Master Volume) */
 	for (i = 0; i < nSamplesToGenerate; i++) {
 		nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
-		MixBuffer[nBufIdx][0] = DmaSnd_IIRfilterL(MixBuffer[nBufIdx][0]);
-		MixBuffer[nBufIdx][1] = DmaSnd_IIRfilterR(MixBuffer[nBufIdx][1]);
+		MixBuffer[nBufIdx][0] = DmaSnd_IIRfilterL( Subsonic_IIR_HPF_Left(MixBuffer[nBufIdx][0]) );
+		MixBuffer[nBufIdx][1] = DmaSnd_IIRfilterR( Subsonic_IIR_HPF_Right(MixBuffer[nBufIdx][1]) );
  	}
 }
 
@@ -873,35 +876,35 @@ void DmaSnd_InterruptHandler_Microwire(void)
 	Uint8 i, bit;
 	Uint16 saveData;
 	
+	/* How many cycle was this sound interrupt delayed (>= 0) */
+	microwire.pendingCyclesOver += -INT_CONVERT_FROM_INTERNAL ( PendingInterruptCount , INT_CPU_CYCLE );
 	/* Remove this interrupt from list and re-order */
 	CycInt_AcknowledgeInterrupt();
 
-	--microwire.mwTransferSteps;
-
-	/* Shift data register until it becomes zero. */
-	if (microwire.mwTransferSteps > 1)
+	/* Shift the mask and data according to the number of cycles (8 cycles for a shift) */
+	do
 	{
+		--microwire.mwTransferSteps;
+			/* Shift data register until it becomes zero. */
 		IoMem_WriteWord(0xff8922, microwire.data<<(16-microwire.mwTransferSteps));
+			/* Rotate mask register */
+		IoMem_WriteWord(0xff8924, (microwire.mask<<(16-microwire.mwTransferSteps))
+								|(microwire.mask>>microwire.mwTransferSteps));
+		/* 8 cycles for 1 shift */
+		microwire.pendingCyclesOver -= 8;
 	}
-	else
-	{
-		/* Paradox XMAS 2004 demo continuesly writes to the data
-		 * register, but still expects to read a zero inbetween,
-		 * so we have to output a zero before we're really done
-		 * with the transfer. */
-		IoMem_WriteWord(0xff8922, 0);
-	}
-
-	/* Rotate mask register */
-	IoMem_WriteWord(0xff8924, (microwire.mask<<(16-microwire.mwTransferSteps))
-	                          |(microwire.mask>>microwire.mwTransferSteps));
+	while ((microwire.mwTransferSteps != 0) && (microwire.pendingCyclesOver >= 8) );
 
+	/* Is the transfer finished ? */
 	if (microwire.mwTransferSteps > 0)
 	{
-		CycInt_AddRelativeInterrupt(8, INT_CPU_CYCLE, INTERRUPT_DMASOUND_MICROWIRE);
+		/* No ==> start a new internal interrupt to continue to tranfer the data */
+		microwire.pendingCyclesOver = 8 - microwire.pendingCyclesOver;
+		CycInt_AddRelativeInterrupt(microwire.pendingCyclesOver, INT_CPU_CYCLE, INTERRUPT_DMASOUND_MICROWIRE);
 	}
-	else {
-		/* Decode the address + command word according to the binary mask */
+	else 
+	{
+		/* Yes : decode the address + command word according to the binary mask */
 		bit = 0;
 		saveData = microwire.data;
 		microwire.data = 0;
@@ -984,7 +987,8 @@ void DmaSnd_MicrowireData_WriteWord(void)
 		microwire.data = IoMem_ReadWord(0xff8922);
 		/* Start shifting events to simulate a microwire transfer */
 		microwire.mwTransferSteps = 16;
-		CycInt_AddRelativeInterrupt(8, INT_CPU_CYCLE, INTERRUPT_DMASOUND_MICROWIRE);
+		microwire.pendingCyclesOver = 8;
+		CycInt_AddRelativeInterrupt(microwire.pendingCyclesOver, INT_CPU_CYCLE, INTERRUPT_DMASOUND_MICROWIRE);
 	}
 
 	LOG_TRACE(TRACE_DMASND, "Microwire data write: 0x%x\n", IoMem_ReadWord(0xff8922));
