@@ -553,7 +553,9 @@ FILE *File_Open(const char *path, const char *mode)
 	/* Open a normal log file */
 	fp = fopen(path, mode);
 	if (!fp)
-		fprintf(stderr, "Can't open file '%s':\n  %s\n", path, strerror(errno));
+		fprintf(stderr, "Can't open file '%s' (wr=%i, rd=%i):\n  %s\n",
+			path, wr, rd, strerror(errno));
+
 	/* printf("'%s' opened in mode '%s'\n", path, mode, fp); */
 	return fp;
 }
