@@ -32,6 +32,7 @@ typedef struct
 typedef struct
 {
   char szTosImageFileName[FILENAME_MAX];
+  bool bPatchTos;
   char szCartridgeImageFileName[FILENAME_MAX];
 } CNF_ROM;
 
@@ -152,7 +153,7 @@ typedef enum
 typedef struct
 {
   bool bAutoInsertDiskB;
-  bool bSlowFloppy;                  /* true to slow down FDC emulation */
+  bool FastFloppy;			/* true to speed up FDC emulation */
   WRITEPROTECTION nWriteProtection;
   char szDiskZipPath[MAX_FLOPPYDRIVES][FILENAME_MAX];
   char szDiskFileName[MAX_FLOPPYDRIVES][FILENAME_MAX];
@@ -160,16 +161,9 @@ typedef struct
 } CNF_DISKIMAGE;
 
 
-/* Hard drives configuration */
-#define MAX_HARDDRIVES  23
-
-typedef enum
-{
-  DRIVE_C,
-  DRIVE_D,
-  DRIVE_E,
-  DRIVE_F
-} DRIVELETTER;
+/* Hard drives configuration: C: - Z: */
+#define MAX_HARDDRIVES  24
+#define DRIVE_C 0
 
 typedef struct
 {
@@ -212,6 +206,7 @@ typedef struct
   int nFrameSkips;
   bool bFullScreen;
   bool bKeepResolution;
+  bool bKeepResolutionST;
   bool bAllowOverscan;
   bool bAspectCorrect;
   bool bUseExtVdiResolutions;
@@ -223,6 +218,7 @@ typedef struct
   bool bShowStatusbar;
   bool bShowDriveLed;
   bool bCrop;
+  bool bForceMax;
   int nMaxWidth;
   int nMaxHeight;
 } CNF_SCREEN;
@@ -283,8 +279,9 @@ typedef struct
   DSPTYPE nDSPType;               /* how to "emulate" DSP */
   bool bRealTimeClock;
   bool bPatchTimerD;
+  bool bFastBoot;                 /* Enable to patch TOS for fast boot */
   bool bFastForward;
-  
+
 #if ENABLE_WINUAE_CPU
   bool bAddressSpace24;
   bool bCycleExactCpu;
