@@ -20,7 +20,8 @@ typedef enum
   INTERRUPT_MFP_TIMERC,
   INTERRUPT_MFP_TIMERD,
   INTERRUPT_IKBD_RESETTIMER,
-  INTERRUPT_IKBD_ACIA,
+  INTERRUPT_IKBD_ACIA_TX,
+  INTERRUPT_IKBD_ACIA_RX,
   INTERRUPT_IKBD_MFP,
   INTERRUPT_IKBD_AUTOSEND,
   INTERRUPT_DMASOUND_MICROWIRE,
