@@ -34,6 +34,7 @@ const char IoMem_fileid[] = "Hatari ioMem.c : " __DATE__ " " __TIME__;
 #include "configuration.h"
 #include "ioMem.h"
 #include "ioMemTables.h"
+#include "memorySnapShot.h"
 #include "m68000.h"
 #include "sysdeps.h"
 
@@ -46,6 +47,21 @@ Uint32 IoAccessBaseAddress;                           /* Stores the base address
 Uint32 IoAccessCurrentAddress;                        /* Current byte address while handling WORD and LONG accesses */
 static int nBusErrorAccesses;                         /* Needed to count bus error accesses */
 
+/* Falcon bus mode (Falcon STe compatible bus or Falcon only bus) */
+static enum {
+	STE_BUS_COMPATIBLE,
+	FALCON_ONLY_BUS
+} falconBusMode;
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Save/Restore snapshot of local variables ('MemorySnapShot_Store' handles type)
+ */
+void IoMem_MemorySnapShot_Capture(bool bSave)
+{
+	/* Save/Restore details */
+	MemorySnapShot_Store(&falconBusMode, sizeof(falconBusMode));
+}
 
 /*-----------------------------------------------------------------------*/
 /**
@@ -86,12 +102,50 @@ void IoMem_Init(void)
 	/* Set default IO access handler (-> bus error) */
 	IoMem_SetBusErrorRegion(0xff8000, 0xffffff);
 
+
+	/* Initialize STe bus specific registers for Falcon in FALCON STe compatible bus mode */
+	if ((ConfigureParams.System.nMachineType == MACHINE_FALCON) && (falconBusMode == STE_BUS_COMPATIBLE)) {
+		for (addr = 0xff8000; addr < 0xffd426; addr++)
+		{
+			if ( ((addr >= 0xff8002) && (addr < 0xff8006)) ||
+				 ((addr >= 0xff8008) && (addr < 0xff800c)) ||
+				 ((addr >= 0xff800e) && (addr < 0xff8060)) ||
+				 ((addr >= 0xff8064) && (addr < 0xff8200)) ||
+				 ((addr >= 0xff82c4) && (addr < 0xff8400)) ||
+				  (addr == 0xff8560)                       ||
+				  (addr == 0xff8564)                       ||
+				 ((addr >= 0xff8804) && (addr < 0xff8900)) ||
+				 ((addr >= 0xff8964) && (addr < 0xff8970)) ||
+				 ((addr >= 0xff8c00) && (addr < 0xff8c80)) ||
+				 ((addr >= 0xff8c88) && (addr < 0xff8d00)) ||
+				 ((addr >= 0xff9000) && (addr < 0xff9200)) ||
+				 ((addr >= 0xff9204) && (addr < 0xff9206)) ||
+				 ((addr >= 0xff9207) && (addr < 0xff9210)) ||
+				 ((addr >= 0xff9218) && (addr < 0xff9220)) ||
+				 ((addr >= 0xff9224) && (addr < 0xff9800)) ||
+				 ((addr >= 0xff9c00) && (addr < 0xffa000)) ||
+				 ((addr >= 0xffa200) && (addr < 0xffa208)) ||
+				  (addr == 0xffc020)                       ||
+				  (addr == 0xffc021)                       ||
+				  (addr == 0xffd020)                       ||
+				  (addr == 0xffd021)                       ||
+				  (addr == 0xffd420)                       ||
+				  (addr == 0xffd421)                       ||
+				  (addr == 0xffd425)
+			  ) {
+					pInterceptReadTable[addr - 0xff8000] = IoMem_VoidRead;      /* For 'read' */
+					pInterceptWriteTable[addr - 0xff8000] = IoMem_VoidWrite;    /* and 'write' */
+ 			    }
+		}
+	}
+
 	switch (ConfigureParams.System.nMachineType)
 	{
 		case MACHINE_ST:  pInterceptAccessFuncs = IoMemTable_ST; break;
 		case MACHINE_STE: pInterceptAccessFuncs = IoMemTable_STE; break;
 		case MACHINE_TT: pInterceptAccessFuncs = IoMemTable_TT; break;
 		case MACHINE_FALCON: pInterceptAccessFuncs = IoMemTable_Falcon; break;
+		default: abort(); /* bug */
 	}
 
 	/* Now set the correct handlers */
@@ -154,7 +208,7 @@ void IoMem_Init(void)
 		}
 	}
 
-	/* Initialize PSG shadow registers */
+	/* Initialize PSG shadow registers for ST, STe, TT machines */
 	if (ConfigureParams.System.nMachineType != MACHINE_FALCON)
 	{
 		for (addr = 0xff8804; addr < 0xff8900; addr++)
@@ -162,10 +216,34 @@ void IoMem_Init(void)
 			pInterceptReadTable[addr - 0xff8000] = pInterceptReadTable[(addr & 0xfff803) - 0xff8000];
 			pInterceptWriteTable[addr - 0xff8000] = pInterceptWriteTable[(addr & 0xfff803) - 0xff8000];
 		}
-	
+	}
+	else {
+		/* Initialize PSG shadow registers for Falcon machine when in STe bus compatibility mode */
+		if (falconBusMode == STE_BUS_COMPATIBLE) {
+			for (addr = 0xff8804; addr < 0xff8900; addr++)
+			{
+				pInterceptReadTable[addr - 0xff8000] = IoMem_VoidRead;     /* For 'read' */
+				pInterceptWriteTable[addr - 0xff8000] = IoMem_VoidWrite;   /* and 'write' */
+			}
+
+		}
 	}
 }
 
+/*-----------------------------------------------------------------------*/
+/**
+ * This function is called to fix falconBusMode (0 = Falcon STe bus compatibility, other value = Falcon only bus compatibility)
+ * This value comes from register $ff8007.b (Bit 5) and is called by ioMemTabFalcon
+ */
+void IoMem_Init_FalconInSTeBuscompatibilityMode(Uint8 value)
+{
+	if (value == 0)
+		falconBusMode = STE_BUS_COMPATIBLE;
+	else
+		falconBusMode = FALCON_ONLY_BUS;
+
+	IoMem_Init();
+}
 
 /*-----------------------------------------------------------------------*/
 /**
@@ -189,7 +267,7 @@ uae_u32 IoMem_bget(uaecptr addr)
 	if (addr < 0xff8000)
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr, 1);
+		M68000_BusError(addr, BUS_ERROR_READ);
 		return -1;
 	}
 
@@ -203,7 +281,7 @@ uae_u32 IoMem_bget(uaecptr addr)
 	/* Check if we read from a bus-error region */
 	if (nBusErrorAccesses == 1)
 	{
-		M68000_BusError(addr, 1);
+		M68000_BusError(addr, BUS_ERROR_READ);
 		return -1;
 	}
 
@@ -229,7 +307,7 @@ uae_u32 IoMem_wget(uaecptr addr)
 	if (addr < 0xff8000)
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr, 1);
+		M68000_BusError(addr, BUS_ERROR_READ);
 		return -1;
 	}
 	if (addr > 0xfffffe)
@@ -255,7 +333,7 @@ uae_u32 IoMem_wget(uaecptr addr)
 	/* Check if we completely read from a bus-error region */
 	if (nBusErrorAccesses == 2)
 	{
-		M68000_BusError(addr, 1);
+		M68000_BusError(addr, BUS_ERROR_READ);
 		return -1;
 	}
 
@@ -281,7 +359,7 @@ uae_u32 IoMem_lget(uaecptr addr)
 	if (addr < 0xff8000)
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr, 1);
+		M68000_BusError(addr, BUS_ERROR_READ);
 		return -1;
 	}
 	if (addr > 0xfffffc)
@@ -319,7 +397,7 @@ uae_u32 IoMem_lget(uaecptr addr)
 	/* Check if we completely read from a bus-error region */
 	if (nBusErrorAccesses == 4)
 	{
-		M68000_BusError(addr, 1);
+		M68000_BusError(addr, BUS_ERROR_READ);
 		return -1;
 	}
 
@@ -344,7 +422,7 @@ void IoMem_bput(uaecptr addr, uae_u32 val)
 	if (addr < 0xff8000 || !regs.s)
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr, 0);
+		M68000_BusError(addr, BUS_ERROR_WRITE);
 		return;
 	}
 
@@ -360,7 +438,7 @@ void IoMem_bput(uaecptr addr, uae_u32 val)
 	/* Check if we wrote to a bus-error region */
 	if (nBusErrorAccesses == 1)
 	{
-		M68000_BusError(addr, 0);
+		M68000_BusError(addr, BUS_ERROR_WRITE);
 	}
 }
 
@@ -380,7 +458,7 @@ void IoMem_wput(uaecptr addr, uae_u32 val)
 	if (addr < 0x00ff8000 || !regs.s)
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr, 0);
+		M68000_BusError(addr, BUS_ERROR_WRITE);
 		return;
 	}
 	if (addr > 0xfffffe)
@@ -408,7 +486,7 @@ void IoMem_wput(uaecptr addr, uae_u32 val)
 	/* Check if we wrote to a bus-error region */
 	if (nBusErrorAccesses == 2)
 	{
-		M68000_BusError(addr, 0);
+		M68000_BusError(addr, BUS_ERROR_WRITE);
 	}
 }
 
@@ -428,7 +506,7 @@ void IoMem_lput(uaecptr addr, uae_u32 val)
 	if (addr < 0xff8000 || !regs.s)
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr, 0);
+		M68000_BusError(addr, BUS_ERROR_WRITE);
 		return;
 	}
 	if (addr > 0xfffffc)
@@ -468,7 +546,7 @@ void IoMem_lput(uaecptr addr, uae_u32 val)
 	/* Check if we wrote to a bus-error region */
 	if (nBusErrorAccesses == 4)
 	{
-		M68000_BusError(addr, 0);
+		M68000_BusError(addr, BUS_ERROR_WRITE);
 	}
 }
 
@@ -537,6 +615,26 @@ void IoMem_VoidRead(void)
 	}
 }
 
+/*-------------------------------------------------------------------------*/
+/**
+ * This is the same function as IoMem_VoidRead, but for IO registers that
+ * return 0x00 instead of 0xff when read (this is the case for some video
+ * registers on STE, Falcon, ...)
+ */
+void IoMem_VoidRead_00(void)
+{
+	Uint32 a;
+
+	/* handler is probably called only once, so we have to take care of the neighbour "void IO registers" */
+	for (a = IoAccessBaseAddress; a < IoAccessBaseAddress + nIoMemAccessSize; a++)
+	{
+		if (pInterceptReadTable[a - 0xff8000] == IoMem_VoidRead_00)
+		{
+			IoMem[a] = 0x00;
+		}
+	}
+}
+
 /*-------------------------------------------------------------------------*/
 /**
  * This is the write handler for the IO memory locations without an assigned
