@@ -66,11 +66,13 @@ enum {
 	OPT_DRIVE_LED,
 	OPT_FORCEBPP,
 	OPT_BORDERS,		/* ST/STE display options */
+	OPT_RESOLUTION_ST,
 	OPT_SPEC512,
 	OPT_ZOOM,
 	OPT_RESOLUTION,		/* Falcon/TT display options */
 	OPT_MAXWIDTH,
 	OPT_MAXHEIGHT,
+	OPT_FORCE_MAX,
 	OPT_ASPECT,
 	OPT_VDI,		/* VDI options */
 	OPT_VDI_PLANES,
@@ -96,6 +98,7 @@ enum {
 	OPT_DISKA,		/* disk options */
 	OPT_DISKB,
 	OPT_SLOWFLOPPY,
+	OPT_FASTFLOPPY,
 	OPT_WRITEPROT_FLOPPY,
 	OPT_WRITEPROT_HD,
 	OPT_HARDDRIVE,
@@ -103,9 +106,10 @@ enum {
 	OPT_IDEMASTERHDIMAGE,
 	OPT_IDESLAVEHDIMAGE,
 	OPT_MEMSIZE,		/* memory options */
-	OPT_TOS,
-	OPT_CARTRIDGE,
 	OPT_MEMSTATE,
+	OPT_TOS,		/* ROM options */
+	OPT_PATCHTOS,
+	OPT_CARTRIDGE,
 	OPT_CPULEVEL,		/* CPU options */
 	OPT_CPUCLOCK,
 	OPT_COMPATIBLE,
@@ -119,12 +123,13 @@ enum {
 	OPT_MACHINE,		/* system options */
 	OPT_BLITTER,
 	OPT_DSP,
-	OPT_MICROPHONE,
+	OPT_TIMERD,
+	OPT_FASTBOOT,
+	OPT_RTC,
+	OPT_MICROPHONE,		/* sound options */
 	OPT_SOUND,
 	OPT_SOUNDBUFFERSIZE,
 	OPT_YM_MIXING,
-	OPT_TIMERD,
-	OPT_RTC,
 	OPT_DEBUG,		/* debug options */
 	OPT_BIOSINTERCEPT,
 	OPT_TRACE,
@@ -189,6 +194,8 @@ static const opt_t HatariOptions[] = {
 	{ OPT_HEADER, NULL, NULL, NULL, "ST/STE specific display" },
 	{ OPT_BORDERS, NULL, "--borders",
 	  "<bool>", "Show screen borders (for overscan demos etc)" },
+	{ OPT_RESOLUTION_ST, NULL, "--desktop-st",
+	  "<bool>", "Keep desktop resolution on fullscreen (no zoom)" },
 	{ OPT_SPEC512, NULL, "--spec512",
 	  "<x>", "Spec512 palette threshold (0 <= x <= 512, 0=disable)" },
 	{ OPT_ZOOM, "-z", "--zoom",
@@ -201,6 +208,8 @@ static const opt_t HatariOptions[] = {
 	  "<x>", "Maximum window width for zooming" },
 	{ OPT_MAXHEIGHT, NULL, "--max-height",
 	  "<x>", "Maximum window height for zooming" },
+	{ OPT_FORCE_MAX, NULL, "--force-max",
+	  "<bool>", "Resolution fixed to given max values" },
 	{ OPT_ASPECT, NULL, "--aspect",
 	  "<bool>", "Monitor aspect ratio correction" },
 
@@ -263,7 +272,9 @@ static const opt_t HatariOptions[] = {
 	{ OPT_DISKB, NULL, "--disk-b",
 	  "<file>", "Set disk image for floppy drive B" },
 	{ OPT_SLOWFLOPPY,   NULL, "--slowfdc",
-	  "<bool>", "Slow down floppy disk access emulation" },
+	  "<bool>", "Slow down floppy disk access emulation (deprecated, use --fastfdc)" },
+	{ OPT_FASTFLOPPY,   NULL, "--fastfdc",
+	  "<bool>", "Speed up floppy disk access emulation (can break some programs)" },
 	{ OPT_WRITEPROT_FLOPPY, NULL, "--protect-floppy",
 	  "<x>", "Write protect floppy image contents (on/off/auto)" },
 	{ OPT_WRITEPROT_HD, NULL, "--protect-hd",
@@ -280,13 +291,17 @@ static const opt_t HatariOptions[] = {
 	{ OPT_HEADER, NULL, NULL, NULL, "Memory" },
 	{ OPT_MEMSIZE,   "-s", "--memsize",
 	  "<x>", "ST RAM size (x = size in MiB from 0 to 14, 0 = 512KiB)" },
-	{ OPT_TOS,       "-t", "--tos",
-	  "<file>", "Use TOS image <file>" },
-	{ OPT_CARTRIDGE, NULL, "--cartridge",
-	  "<file>", "Use ROM cartridge image <file>" },
 	{ OPT_MEMSTATE,   NULL, "--memstate",
 	  "<file>", "Load memory snap-shot <file>" },
-	
+
+	{ OPT_HEADER, NULL, NULL, NULL, "ROM" },
+	{ OPT_TOS,       "-t", "--tos",
+	  "<file>", "Use TOS image <file>" },
+	{ OPT_PATCHTOS, NULL, "--patch-tos",
+	  "<bool>", "Apply TOS patches (experts only, leave it enabled!)" },
+	{ OPT_CARTRIDGE, NULL, "--cartridge",
+	  "<file>", "Use ROM cartridge image <file>" },
+
 	{ OPT_HEADER, NULL, NULL, NULL, "CPU" },
 	{ OPT_CPULEVEL,  NULL, "--cpulevel",
 	  "<x>", "Set the CPU type (x => 680x0) (EmuTOS/TOS 2.06 only!)" },
@@ -316,18 +331,22 @@ static const opt_t HatariOptions[] = {
 	  "<bool>", "Use blitter emulation (ST only)" },
 	{ OPT_DSP,       NULL, "--dsp",
 	  "<x>", "DSP emulation (x = none/dummy/emu, Falcon only)" },
+	{ OPT_TIMERD,    NULL, "--timer-d",
+	  "<bool>", "Patch Timer-D (about doubles ST emulation speed)" },
+	{ OPT_FASTBOOT, NULL, "--fast-boot",
+	  "<bool>", "Patch TOS and memvalid system variables for faster boot" },
+	{ OPT_RTC,    NULL, "--rtc",
+	  "<bool>", "Enable real-time clock" },
+
+	{ OPT_HEADER, NULL, NULL, NULL, "Sound" },
 	{ OPT_MICROPHONE,   NULL, "--mic",
 	  "<bool>", "Enable/disable (Falcon only) microphone" },
 	{ OPT_SOUND,   NULL, "--sound",
 	  "<x>", "Sound frequency (x=off/6000-50066, off=fastest)" },
 	{ OPT_SOUNDBUFFERSIZE,   NULL, "--sound-buffer-size",
-	  "<x>", "Sound buffer size for SDL in ms (x=0/10-100, 0=default)" },
+	  "<x>", "Sound buffer size in ms (x=0/10-100, 0=default)" },
 	{ OPT_YM_MIXING,   NULL, "--ym-mixing",
-	  "<x>", "YM sound mixing method (x=linear/table)" },
-	{ OPT_TIMERD,    NULL, "--timer-d",
-	  "<bool>", "Patch Timer-D (about doubles ST emulation speed)" },
-	{ OPT_RTC,    NULL, "--rtc",
-	  "<bool>", "Enable real-time clock" },
+	  "<x>", "YM sound mixing method (x=linear/table/model)" },
 
 	{ OPT_HEADER, NULL, NULL, NULL, "Debug" },
 	{ OPT_DEBUG,     "-D", "--debug",
@@ -834,18 +853,18 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 		/* last argument can be a non-option */
 		if (argv[i][0] != '-' && i+1 == argc)
 			return Opt_HandleArgument(argv[i]);
-    
+
 		/* WhichOption() checks also that there is an argument,
 		 * so we don't need to check that below
 		 */
 		switch(Opt_WhichOption(argc, argv, i))
 		{
-		
+
 			/* general options */
 		case OPT_HELP:
 			Opt_ShowHelp();
 			return false;
-			
+
 		case OPT_VERSION:
 			Opt_ShowVersion();
 			return false;
@@ -857,7 +876,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 		case OPT_FASTFORWARD:
 			ok = Opt_Bool(argv[++i], OPT_FASTFORWARD, &ConfigureParams.System.bFastForward);
 			break;
-			
+
 		case OPT_CONFIGFILE:
 			i += 1;
 			/* true -> file needs to exist */
@@ -869,7 +888,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				bLoadAutoSave = ConfigureParams.Memory.bAutoSave;
 			}
 			break;
-		
+
 			/* common display options */
 		case OPT_MONO:
 			ConfigureParams.Screen.nMonitorType = MONITOR_TYPE_MONO;
@@ -900,11 +919,11 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			}
 			bLoadAutoSave = false;
 			break;
-			
+
 		case OPT_FULLSCREEN:
 			ConfigureParams.Screen.bFullScreen = true;
 			break;
-			
+
 		case OPT_WINDOW:
 			ConfigureParams.Screen.bFullScreen = false;
 			break;
@@ -912,7 +931,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 		case OPT_GRAB:
 			bGrabMouse = true;
 			break;
-			
+
 		case OPT_FRAMESKIPS:
 			skips = atoi(argv[++i]);
 			if (skips < 0)
@@ -926,15 +945,15 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			}
 			ConfigureParams.Screen.nFrameSkips = skips;
 			break;
-			
+
 		case OPT_STATUSBAR:
 			ok = Opt_Bool(argv[++i], OPT_STATUSBAR, &ConfigureParams.Screen.bShowStatusbar);
 			break;
-			
+
 		case OPT_DRIVE_LED:
 			ok = Opt_Bool(argv[++i], OPT_DRIVE_LED, &ConfigureParams.Screen.bShowDriveLed);
 			break;
-			
+
 		case OPT_FORCEBPP:
 			planes = atoi(argv[++i]);
 			switch(planes)
@@ -952,11 +971,15 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			}
 			ConfigureParams.Screen.nForceBpp = planes;
 			break;
-			
+
 			/* ST/STE display options */
 		case OPT_BORDERS:
 			ok = Opt_Bool(argv[++i], OPT_BORDERS, &ConfigureParams.Screen.bAllowOverscan);
 			break;
+
+		case OPT_RESOLUTION_ST:
+			ok = Opt_Bool(argv[++i], OPT_RESOLUTION_ST, &ConfigureParams.Screen.bKeepResolutionST);
+			break;
 			
 		case OPT_SPEC512:
 			threshold = atoi(argv[++i]);
@@ -998,6 +1021,10 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 		case OPT_MAXHEIGHT:
 			ConfigureParams.Screen.nMaxHeight = atoi(argv[++i]);
 			break;
+
+		case OPT_FORCE_MAX:
+			ok = Opt_Bool(argv[++i], OPT_FORCE_MAX, &ConfigureParams.Screen.bForceMax);
+			break;
 			
 		case OPT_ASPECT:
 			ok = Opt_Bool(argv[++i], OPT_ASPECT, &ConfigureParams.Screen.bAspectCorrect);
@@ -1122,7 +1149,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				return Opt_ShowError(OPT_JOYSTICK0+port, argv[i], "Invalid joystick type");
 			}
 			break;
-			
+
 		case OPT_PRINTER:
 			i += 1;
 			/* "none" can be used to disable printer */
@@ -1130,7 +1157,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 					argv[i], sizeof(ConfigureParams.Printer.szPrintToFileName),
 					&ConfigureParams.Printer.bEnablePrinting);
 			break;
-			
+
 		case OPT_MIDI_IN:
 			i += 1;
 			ok = Opt_StrCpy(OPT_MIDI_IN, true, ConfigureParams.Midi.sMidiInFileName,
@@ -1180,7 +1207,12 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			break;
 
 		case OPT_SLOWFLOPPY:
-			ok = Opt_Bool(argv[++i], OPT_SLOWFLOPPY, &ConfigureParams.DiskImage.bSlowFloppy);
+			i++;
+			fprintf(stderr, "\nWarning: --slowfdc is not supported anymore, use --fastfdc\n\n");
+			break;
+
+		case OPT_FASTFLOPPY:
+			ok = Opt_Bool(argv[++i], OPT_FASTFLOPPY, &ConfigureParams.DiskImage.FastFloppy);
 			break;
 
 		case OPT_WRITEPROT_FLOPPY:
@@ -1229,7 +1261,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				bLoadAutoSave = false;
 			}
 			break;
-			
+
 		case OPT_IDEMASTERHDIMAGE:
 			i += 1;
 			ok = Opt_StrCpy(OPT_IDEMASTERHDIMAGE, true, ConfigureParams.HardDisk.szIdeMasterHardDiskImage,
@@ -1251,7 +1283,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				bLoadAutoSave = false;
 			}
 			break;
-			
+
 			/* Memory options */
 		case OPT_MEMSIZE:
 			memsize = atoi(argv[++i]);
@@ -1262,7 +1294,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			ConfigureParams.Memory.nMemorySize = memsize;
 			bLoadAutoSave = false;
 			break;
-      
+
 		case OPT_TOS:
 			i += 1;
 			ok = Opt_StrCpy(OPT_TOS, true, ConfigureParams.Rom.szTosImageFileName,
@@ -1273,7 +1305,11 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				bLoadAutoSave = false;
 			}
 			break;
-			
+
+		case OPT_PATCHTOS:
+			ok = Opt_Bool(argv[++i], OPT_PATCHTOS, &ConfigureParams.Rom.bPatchTos);
+			break;
+
 		case OPT_CARTRIDGE:
 			i += 1;
 			ok = Opt_StrCpy(OPT_CARTRIDGE, true, ConfigureParams.Rom.szCartridgeImageFileName,
@@ -1296,7 +1332,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				bLoadAutoSave = false;
 			}
 			break;
-			
+
 			/* CPU options */
 		case OPT_CPULEVEL:
 			/* UAE core uses cpu_level variable */
@@ -1308,7 +1344,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			ConfigureParams.System.nCpuLevel = ncpu;
 			bLoadAutoSave = false;
 			break;
-			
+
 		case OPT_CPUCLOCK:
 			cpuclock = atoi(argv[++i]);
 			if(cpuclock != 8 && cpuclock != 16 && cpuclock != 32)
@@ -1318,7 +1354,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			ConfigureParams.System.nCpuFreq = cpuclock;
 			bLoadAutoSave = false;
 			break;
-			
+
 		case OPT_COMPATIBLE:
 			ok = Opt_Bool(argv[++i], OPT_COMPATIBLE, &ConfigureParams.System.bCompatibleCpu);
 			if (ok)
@@ -1326,6 +1362,51 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				bLoadAutoSave = false;
 			}
 			break;
+#if ENABLE_WINUAE_CPU
+		case OPT_CPU_ADDR24:
+			ok = Opt_Bool(argv[++i], OPT_CPU_ADDR24, &ConfigureParams.System.bAddressSpace24);
+			bLoadAutoSave = false;
+			break;
+
+		case OPT_CPU_CYCLE_EXACT:
+			ok = Opt_Bool(argv[++i], OPT_CPU_CYCLE_EXACT, &ConfigureParams.System.bCycleExactCpu);
+			bLoadAutoSave = false;
+			break;
+
+		case OPT_FPU_TYPE:
+			i += 1;
+			if (strcasecmp(argv[i], "none") == 0)
+			{
+				ConfigureParams.System.n_FPUType = FPU_NONE;
+			}
+			else if (strcasecmp(argv[i], "68881") == 0)
+			{
+				ConfigureParams.System.n_FPUType = FPU_68881;
+			}
+			else if (strcasecmp(argv[i], "68882") == 0)
+			{
+				ConfigureParams.System.n_FPUType = FPU_68882;
+			}
+			else if (strcasecmp(argv[i], "internal") == 0)
+			{
+				ConfigureParams.System.n_FPUType = FPU_CPU;
+			}
+			else
+			{
+				return Opt_ShowError(OPT_FPU_TYPE, argv[i], "Unknown FPU type");
+			}
+			bLoadAutoSave = false;
+			break;
+
+		case OPT_FPU_COMPATIBLE:
+			ok = Opt_Bool(argv[++i], OPT_FPU_COMPATIBLE, &ConfigureParams.System.bCompatibleFPU);
+			break;
+
+		case OPT_MMU:
+			ok = Opt_Bool(argv[++i], OPT_MMU, &ConfigureParams.System.bMMU);
+			bLoadAutoSave = false;
+			break;
+#endif
 
 			/* system options */
 		case OPT_MACHINE:
@@ -1363,22 +1444,25 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			}
 			bLoadAutoSave = false;
 			break;
-			
+
 		case OPT_BLITTER:
 			ok = Opt_Bool(argv[++i], OPT_BLITTER, &ConfigureParams.System.bBlitter);
 			if (ok)
 			{
 				bLoadAutoSave = false;
 			}
-			break;			
-			
+			break;
+
 		case OPT_TIMERD:
 			ok = Opt_Bool(argv[++i], OPT_TIMERD, &ConfigureParams.System.bPatchTimerD);
-			break;			
-			
+			break;
+		case OPT_FASTBOOT:
+			ok = Opt_Bool(argv[++i], OPT_FASTBOOT, &ConfigureParams.System.bFastBoot);
+			break;
+
 		case OPT_RTC:
 			ok = Opt_Bool(argv[++i], OPT_RTC, &ConfigureParams.System.bRealTimeClock);
-			break;			
+			break;
 
 		case OPT_DSP:
 			i += 1;
@@ -1405,51 +1489,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			bLoadAutoSave = false;
 			break;
 
-#if ENABLE_WINUAE_CPU
-		case OPT_CPU_ADDR24:
-			ok = Opt_Bool(argv[++i], OPT_CPU_ADDR24, &ConfigureParams.System.bAddressSpace24);
-			bLoadAutoSave = false;
-			break;			
-
-		case OPT_CPU_CYCLE_EXACT:
-			ok = Opt_Bool(argv[++i], OPT_CPU_CYCLE_EXACT, &ConfigureParams.System.bCycleExactCpu);
-			bLoadAutoSave = false;
-			break;			
-
-		case OPT_FPU_TYPE:
-			i += 1;
-			if (strcasecmp(argv[i], "none") == 0)
-			{
-				ConfigureParams.System.n_FPUType = FPU_NONE;
-			}
-			else if (strcasecmp(argv[i], "68881") == 0)
-			{
-				ConfigureParams.System.n_FPUType = FPU_68881;
-			}
-			else if (strcasecmp(argv[i], "68882") == 0)
-			{
-				ConfigureParams.System.n_FPUType = FPU_68882;
-			}
-			else if (strcasecmp(argv[i], "internal") == 0)
-			{
-				ConfigureParams.System.n_FPUType = FPU_CPU;
-			}
-			else
-			{
-				return Opt_ShowError(OPT_FPU_TYPE, argv[i], "Unknown FPU type");
-			}
-			bLoadAutoSave = false;
-			break;
-
-		case OPT_FPU_COMPATIBLE:
-			ok = Opt_Bool(argv[++i], OPT_FPU_COMPATIBLE, &ConfigureParams.System.bCompatibleFPU);
-			break;			
-
-		case OPT_MMU:
-			ok = Opt_Bool(argv[++i], OPT_MMU, &ConfigureParams.System.bMMU);
-			bLoadAutoSave = false;
-			break;			
-#endif
+			/* sound options */
 		case OPT_YM_MIXING:
 			i += 1;
 			if (strcasecmp(argv[i], "linear") == 0)
@@ -1460,6 +1500,10 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			{
 				ConfigureParams.Sound.YmVolumeMixing = YM_TABLE_MIXING;
 			}
+			else if (strcasecmp(argv[i], "model") == 0)
+			{
+				ConfigureParams.Sound.YmVolumeMixing = YM_MODEL_MIXING;
+			}
 			else
 			{
 				return Opt_ShowError(OPT_YM_MIXING, argv[i], "Unknown YM mixing method");
@@ -1498,7 +1542,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			
 		case OPT_MICROPHONE:
 			ok = Opt_Bool(argv[++i], OPT_MICROPHONE, &ConfigureParams.Sound.bEnableMicrophone);
-			break;			
+			break;
 
 		case OPT_KEYMAPFILE:
 			i += 1;
