@@ -270,8 +270,11 @@ static void TOS_FixRom(void)
 			{
 				/* Only apply the patch if it is really needed: */
 				if (pPatch->Flags == TP_ALWAYS
-				    || (pPatch->Flags == TP_HDIMAGE_OFF && !ACSI_EMU_ON && !ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage)
-				    || (pPatch->Flags == TP_ANTI_STE && ConfigureParams.System.nMachineType == MACHINE_ST))
+				    || (pPatch->Flags == TP_HDIMAGE_OFF && !ACSI_EMU_ON
+				        && !ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage
+				        && ConfigureParams.System.bFastBoot)
+				    || (pPatch->Flags == TP_ANTI_STE
+				        && ConfigureParams.System.nMachineType == MACHINE_ST))
 				{
 					/* Now we can really apply the patch! */
 					Log_Printf(LOG_DEBUG, "Applying TOS patch '%s'.\n", pPatch->pszName);
@@ -474,9 +477,11 @@ static void TOS_CheckSysConfig(void)
 		ConfigureParams.System.nCpuLevel = 3;
 		M68000_CheckCpuSettings();
 	}
-	else if (TosVersion <= 0x0104 && ConfigureParams.System.nCpuLevel > 0)
+	else if (TosVersion <= 0x0104 &&
+	         (ConfigureParams.System.nCpuLevel > 0 || ConfigureParams.System.nMachineType != MACHINE_ST))
 	{
-		Log_AlertDlg(LOG_ERROR, "TOS versions <= 1.4 work only with a 68000 CPU.\n"
+		Log_AlertDlg(LOG_ERROR, "TOS versions <= 1.4 work only in\n"
+		             "ST mode and with a 68000 CPU.\n"
 		             " ==> Switching to ST mode now.\n");
 		IoMem_UnInit();
 		ConfigureParams.System.nMachineType = MACHINE_ST;
@@ -489,7 +494,7 @@ static void TOS_CheckSysConfig(void)
 	else if ((TosVersion < 0x0300 && ConfigureParams.System.nMachineType == MACHINE_FALCON)
 	         || (TosVersion < 0x0200 && ConfigureParams.System.nMachineType == MACHINE_TT))
 	{
-		Log_AlertDlg(LOG_ERROR, "This TOS versions does not work in TT/Falcon mode.\n"
+		Log_AlertDlg(LOG_ERROR, "This TOS version does not work in TT/Falcon mode.\n"
 		             " ==> Switching to STE mode now.\n");
 		IoMem_UnInit();
 		ConfigureParams.System.nMachineType = MACHINE_STE;
@@ -501,11 +506,16 @@ static void TOS_CheckSysConfig(void)
 	}
 	else if (TosVersion >= 0x0300 && ConfigureParams.System.nCpuLevel < 2)
 	{
-		Log_AlertDlg(LOG_ERROR, "This TOS versions requires a CPU >= 68020.\n"
+		Log_AlertDlg(LOG_ERROR, "This TOS version requires a CPU >= 68020.\n"
 		             " ==> Switching to 68020 mode now.\n");
 		ConfigureParams.System.nCpuLevel = 2;
 		M68000_CheckCpuSettings();
 	}
+	if (TosVersion < 0x0104 && ConfigureParams.HardDisk.bUseHardDiskDirectories)
+	{
+		Log_AlertDlg(LOG_ERROR, "Please use at least TOS v1.04 for the HD directory emulation "
+			     "(all required GEMDOS functionality isn't completely emulated for this TOS version).");
+	}
 }
 
 
@@ -616,8 +626,14 @@ int TOS_LoadImage(void)
 	}
 
 	/* Fix TOS image, modify code for emulation */
-	if (!bIsEmuTOS)
+	if (ConfigureParams.Rom.bPatchTos && !bIsEmuTOS)
+	{
 		TOS_FixRom();
+	}
+	else
+	{
+		Log_Printf(LOG_DEBUG, "Skipped TOS patches.\n");
+	}
 
 	/* Set connected devices, memory configuration, etc. */
 	STMemory_SetDefaultConfig();
